//$Id: LongType.java,v 1.5.2.4 2003/11/21 16:06:40 oneovthafew Exp $
package net.sf.hibernate.type;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

/**
 * <tt>long</tt>: A type that maps an SQL BIGINT to a Java Long.
 * @author Gavin King
 */
public class LongType extends PrimitiveType implements DiscriminatorType, VersionType {
	
	private static final Long ZERO = new Long(0);
	
	public Object get(ResultSet rs, String name) throws SQLException {
		return new Long( rs.getLong(name) );
	}
	
	public Class getPrimitiveClass() {
		return long.class;
	}
	
	public Class getReturnedClass() {
		return Long.class;
	}
	
	public void set(PreparedStatement st, Object value, int index)
	throws SQLException {
		
		st.setLong( index, ( (Long) value ).longValue() );
	}
	
	public int sqlType() {
		return Types.BIGINT;
	}
	
	public String getName() { return "long"; }
	
	public Object stringToObject(String xml) throws Exception {
		return new Long(xml);
	}
	
	public Object next(Object current) {
		return new Long( ( (Long) current ).longValue() + 1 );
	}
	
	public Object seed() {
		return ZERO;
	}
	
	public String objectToSQLString(Object value) throws Exception {
		return value.toString();
	}

	public Object fromString(String xml) {
		return new Long(xml);
	}
	
	
}





